#include "squidwall.h"
#include "md5.h"
#include "sha1.h"
#include "hextab.h"

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/**
 * url blacklist checking
 */

int check_blist(void)
{
	struct squidwall_string *string;
	struct squidwall_regex *regex;
	struct named_regex *nr;
	struct list_head *l;
	unsigned int j, k;
	char *file, *str, *e;
	char md5_url[33], sha1_url[41];
	unsigned char digest[20];
	MD5_CTX md5;
	SHA1_CTX sha1;

	debug_s3(DEBUG_CHECK_BLIST, "check_blist() 1. url.host.s(", url_host.s, ")");

	/* setup the md5 hash */
	if (ufilter_md5_user || ufilter_md5_host) {
		MD5_Init(&md5);
		MD5_Update(&md5, url_host.s, strlen(url_host.s));
		MD5_Final(digest, &md5);
		e=md5_url;
		for (j=0; j<16; j++) {
			*e = hextab[digest[j]/16]; ++e;
			*e = hextab[digest[j]%16]; ++e;
		}
		*e=0;
	}
	debug_s3(DEBUG_CHECK_BLIST, "check_blist() 2. md5url(", md5_url, ")");

	/* test the md5 lists (host acl's) */
	if (ufilter_md5_host)
	for (j=0; j < ufilter_md5_host->len; j += str_len(ufilter_md5_host->s+j)+1)
	list_for_each(l, &ufilter_md5) {
		string = list_entry(l, struct squidwall_string, list);
		file = ufilter_md5_host->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(md5_url, str)) {
				if(!stralloc_copy(&filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&filter, " / ")) die_nomem();
				if(!stralloc_cats(&filter, str)) die_nomem();
				filter.s[filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the md5 lists (user acl's) */
	if (ufilter_md5_user)
	for (j=0; j < ufilter_md5_user->len; j += str_len(ufilter_md5_user->s+j)+1)
	list_for_each(l, &ufilter_md5) {
		string = list_entry(l, struct squidwall_string, list);
		file = ufilter_md5_user->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(md5_url, str)) {
				if(!stralloc_copy(&filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&filter, " / ")) die_nomem();
				if(!stralloc_cats(&filter, str)) die_nomem();
				filter.s[filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* setup the sha1 hash */
	if (ufilter_sha1_user || ufilter_sha1_host) {
		SHA1_Init(&sha1);
		SHA1_Update(&sha1, url_host.s, strlen(url_host.s));
		SHA1_Final(digest, &sha1);
		e=sha1_url;
		for (j=0; j<20; j++) {
			*e = hextab[digest[j]/20]; ++e;
			*e = hextab[digest[j]%20]; ++e;
		}
		*e=0;
	}

	/* test the sha1 lists (host acl's) */
	if (ufilter_sha1_host)
	for (j=0; j < ufilter_sha1_host->len; j += str_len(ufilter_sha1_host->s+j)+1)
	list_for_each(l, &ufilter_sha1) {
		string = list_entry(l, struct squidwall_string, list);
		file = ufilter_sha1_host->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(sha1_url, str)) {
				if(!stralloc_copy(&filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&filter, " / ")) die_nomem();
				if(!stralloc_cats(&filter, str)) die_nomem();
				filter.s[filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the sha1 lists (user acl's) */
	if (ufilter_sha1_user)
	for (j=0; j < ufilter_sha1_user->len; j += str_len(ufilter_sha1_user->s+j)+1)
	list_for_each(l, &ufilter_sha1) {
		string = list_entry(l, struct squidwall_string, list);
		file = ufilter_sha1_user->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(sha1_url, str)) {
				if(!stralloc_copy(&filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&filter, " / ")) die_nomem();
				if(!stralloc_cats(&filter, str)) die_nomem();
				filter.s[filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the url blacklists (host acl's) */
	if (ufilter_host)
	for (j=0; j < ufilter_host->len; j += str_len(ufilter_host->s+j)+1)
	list_for_each(l, &ufilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = ufilter_host->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(squid_url, str)) {
				if(!stralloc_copy(&filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&filter, " / ")) die_nomem();
				if(!stralloc_cats(&filter, str)) die_nomem();
				filter.s[filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the url blacklists (user acl's) */
	if (ufilter_user)
	for (j=0; j < ufilter_user->len; j += str_len(ufilter_user->s+j)+1)
	list_for_each(l, &ufilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = ufilter_user->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(squid_url, str)) {
				if(!stralloc_copy(&filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&filter, " / ")) die_nomem();
				if(!stralloc_cats(&filter, str)) die_nomem();
				filter.s[filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the url regex blacklists (host acl's) */
	if (urfilter_host && urfilter_host->len)
	for (j=0; j < urfilter_host->len; j += str_len(urfilter_host->s+j)+1)
	list_for_each(l, &urfilter) {
		file = urfilter_host->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, squid_url, 0, 0, nr->eflags) == 0) {
				if(!stralloc_copy(&filter, &regex->filename)) die_nomem();
				if(!stralloc_cats(&filter, " / ")) die_nomem();
				if(!stralloc_cat(&filter, &nr->n)) die_nomem();
				filter.s[filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	/* test the url regex blacklists (user acl's) */
	if (urfilter_user && urfilter_user->len)
	for (j=0; j < urfilter_user->len; j += str_len(urfilter_user->s+j)+1)
	list_for_each(l, &urfilter) {
		file = urfilter_user->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(regex_t)); k++) {
			nr = array_get(&regex->regex, sizeof(regex_t), k);
			if (regexec(&nr->r, squid_url, 0, 0, nr->eflags) == 0) {
				if(!stralloc_copy(&filter, &regex->filename)) die_nomem();
				if(!stralloc_cats(&filter, " / ")) die_nomem();
				if(!stralloc_cat(&filter, &nr->n)) die_nomem();
				filter.s[filter.len]=0;
				return ACL_DENY;
			}
		}
	}

	return ACL_ALLOW;
}
