#include "squidwall.h"

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* buffers for proxy connection */
static char proxy_oBUF[BUFSIZE]; buffer proxy_out;
static char proxy_iBUF[BUFSIZE]; buffer proxy_in;
int64 fd_proxy;

unsigned int pfilter_current;
stralloc pfilter_matches;

int cfilter_todo;
int pfilter_todo;

/* internal functions */
static int check_cont_hdr(array *a, char *line);
static int check_cont_line(char *line);

#define SCAN_UNKNOWN 1
#define SCAN_YES     2
#define SCAN_NO      3
static int check_cont_hdr(array *a, char *line)
{
	struct squidwall_regex *regex;
	struct named_regex *nr;
	int i, j;

	/* last http header was send, no scanning of this content is wanted! */
	if (str_len(line) == 0) {
		debug_s3(DEBUG_CHECK_CONT,
			"check_hdr: SCAN_NO @ line(", line, ")");
		return SCAN_NO;
	}

	for (i=0; i < array_length(a, sizeof(struct squidwall_regex)); i++) {
		regex = array_get(a, sizeof(struct squidwall_regex), i);
		for (j=0; j < array_length(&regex->regex, sizeof(struct named_regex)); j++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), j);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				debug_s3(DEBUG_CHECK_CONT,
					"check_hdr: SCAN_YES @ line(", line, ")");
				return SCAN_YES;
			}
		}
	}

	debug_s3(DEBUG_CHECK_CONT,
		"check_hdr: SCAN_UNKNOWN @ line(", line, ")");

	return SCAN_UNKNOWN;
}

static int check_cont_line(char *line)
{
	struct squidwall_string *string;
	struct squidwall_phrase *phrase;
	struct squidwall_regex *regex;
	struct named_regex *nr;
	struct list_head *l;
	char *file, *str, num[FMT_ULONG];
	unsigned int j, k, k2, *val;

	if (cfilter_todo == SCAN_YES)
	if (cfilter_host && cfilter_host->len)
	for (j=0; j < cfilter_host->len; j += str_len(cfilter_host->s+j)+1)
	list_for_each(l, &cfilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = cfilter_host->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(line, str)) {
				if (!stralloc_copy(&filter, &string->filename)) die_nomem();
				if (!stralloc_cats(&filter, " / ")) die_nomem();
				if (!stralloc_cats(&filter, str)) die_nomem();
				filter.s[filter.len]=0;
				redir_site(REDIR_TYPE_CONT);
				return ACL_DENY;
			}
		}
	}

	if (cfilter_todo == SCAN_YES)
	if (cfilter_user && cfilter_user->len)
	for (j=0; j < cfilter_user->len; j += str_len(cfilter_user->s+j)+1)
	list_for_each(l, &cfilter) {
		string = list_entry(l, struct squidwall_string, list);
		file = cfilter_user->s+j;
		if (str_equal(file, string->filename.s))
		for (k=0; k < string->strings.len;
				k += str_len(string->strings.s+k)+1) {
			str = string->strings.s+k;
			if (case_strstr(line, str)) {
				if(!stralloc_copy(&filter, &string->filename)) die_nomem();
				if(!stralloc_cats(&filter, " / ")) die_nomem();
				if(!stralloc_cats(&filter, str)) die_nomem();
				filter.s[filter.len]=0;
				redir_site(REDIR_TYPE_CONT);
				return ACL_DENY;
			}
		}
	}

	if (pfilter_todo == SCAN_YES)
	if (pfilter_user && pfilter_user->len)
	for (j=0; j < pfilter_user->len; j += str_len(pfilter_user->s+j)+1)
	list_for_each(l, &pfilter) {
		phrase = list_entry(l, struct squidwall_phrase, list);
		file = pfilter_user->s+j;
		if (str_equal(file, phrase->filename.s))
		for (k=0, k2=0; k < phrase->strings.len;
				k += str_len(phrase->strings.s+k)+1, k2++) {
			str = phrase->strings.s+k;
			val = array_get(&phrase->points, sizeof(unsigned int), k2);
			if (case_strstr(line, str)) {
				num[fmt_ulong(num, *val)]=0;
				if (!stralloc_cat(&pfilter_matches, &phrase->filename)) die_nomem();
				if (!stralloc_cats(&pfilter_matches, " (")) die_nomem();
				if (!stralloc_cats(&pfilter_matches, str)) die_nomem();
				if (!stralloc_cats(&pfilter_matches, " / ")) die_nomem();
				if (!stralloc_cats(&pfilter_matches, num)) die_nomem();
				if (!stralloc_cats(&pfilter_matches, ")<br>")) die_nomem();
				pfilter_current += *val;
				if (pfilter_current >= pfilter_max) {
					if (!stralloc_copy(&filter, &pfilter_matches)) die_nomem();
					redir_site(REDIR_TYPE_CONT);
					filter.s[filter.len]=0;
					return ACL_DENY;
				}
			}
		}
	}

	if (pfilter_todo == SCAN_YES)
	if (pfilter_host && pfilter_host->len)
	for (j=0; j < pfilter_host->len; j += str_len(pfilter_host->s+j)+1)
	list_for_each(l, &pfilter) {
		phrase = list_entry(l, struct squidwall_phrase, list);
		file = pfilter_host->s+j;
		if (str_equal(file, phrase->filename.s))
		for (k=0, k2=0; k < phrase->strings.len;
				k += str_len(phrase->strings.s+k)+1, k2++) {
			str = phrase->strings.s+k;
			val = array_get(&phrase->points, sizeof(unsigned int), k2);
			if (case_strstr(line, str)) {
				num[fmt_ulong(num, *val)]=0;
				if (!stralloc_cat(&pfilter_matches, &phrase->filename)) die_nomem();
				if (!stralloc_cats(&pfilter_matches, " (")) die_nomem();
				if (!stralloc_cats(&pfilter_matches, str)) die_nomem();
				if (!stralloc_cats(&pfilter_matches, " / ")) die_nomem();
				if (!stralloc_cats(&pfilter_matches, num)) die_nomem();
				if (!stralloc_cats(&pfilter_matches, ")<br>")) die_nomem();
				pfilter_current += *val;
				if (pfilter_current >= pfilter_max) {
					if (!stralloc_copy(&filter, &pfilter_matches)) die_nomem();
					filter.s[filter.len]=0;
					redir_site(REDIR_TYPE_CONT);
					return ACL_DENY;
				}
			}
		}
	}

	if (crfilter_host && crfilter_host->len)
	for (j=0; j < crfilter_host->len; j += str_len(crfilter_host->s+j)+1)
	list_for_each(l, &crfilter) {
		file = crfilter_host->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				if (!stralloc_copy(&filter, &regex->filename)) die_nomem();
				if (!stralloc_cats(&filter, " / ")) die_nomem();
				if (!stralloc_cat(&filter, &nr->n)) die_nomem();
				filter.s[filter.len]=0;
				redir_site(REDIR_TYPE_CONT);
				return ACL_DENY;
			}
		}
	}

	if (crfilter_user && crfilter_user->len)
	for (j=0; j < crfilter_user->len; j += str_len(crfilter_user->s+j)+1)
	list_for_each(l, &crfilter) {
		file = crfilter_user->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				if (!stralloc_copy(&filter, &regex->filename)) die_nomem();
				if (!stralloc_cats(&filter, " / ")) die_nomem();
				if (!stralloc_cat(&filter, &nr->n)) die_nomem();
				filter.s[filter.len]=0;
				redir_site(REDIR_TYPE_CONT);
				return ACL_DENY;
			}
		}
	}

	if (bcfilter_host && bcfilter_host->len)
	for (j=0; j < bcfilter_host->len; j += str_len(bcfilter_host->s+j)+1)
	list_for_each(l, &bcfilter) {
		file = bcfilter_host->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				redir_site(REDIR_TYPE_BANNER);
				return ACL_DENY;
			}
		}
	}

	if (bcfilter_user && bcfilter_user->len)
	for (j=0; j < bcfilter_user->len; j += str_len(bcfilter_user->s+j)+1)
	list_for_each(l, &bcfilter) {
		file = bcfilter_user->s+j;
		regex = list_entry(l, struct squidwall_regex, list);
		if (str_equal(file, regex->filename.s))
		for (k=0; k < array_length(&regex->regex, sizeof(struct named_regex)); k++) {
			nr = array_get(&regex->regex, sizeof(struct named_regex), k);
			if (regexec(&nr->r, line, 0, 0, nr->eflags) == 0) {
				redir_site(REDIR_TYPE_BANNER);
				return ACL_DENY;
			}
		}
	}

	return ACL_ALLOW;
}


int check_cont(int todo)
{
	stralloc content;
	size_t len=0;
	char ch;

	if (!str_equal(squid_method, "GET"))
		return ACL_ALLOW;

	stralloc_init(&content);
	stralloc_init(&pfilter_matches);

	if (!stralloc_copys(&clamav, "")) die_nomem();
	if (!stralloc_copys(&filter, "")) die_nomem();
	if (!stralloc_copys(&virus, "")) die_nomem();

	/* connect to clamav */
	if (isset(todo, TODO_VIRUS)) {
		check_clamav_connect();
	}

	/* open connection to proxy */
	fd_proxy=socket_tcp4();
	if (fd_proxy == -1) {
		return panic("socket() for proxy failed");
	}

        socket_connect4(fd_proxy, (char*)&proxy_ip, proxy_port);
	if (!socket_connected(fd_proxy)) {
		close(fd_proxy);
		return panic("connect() to proxy failed");
	}
	if (!io_fd(fd_proxy)) return panic("io_fd() failed");

	buffer_init(&proxy_in,   (int(*)())read, fd_proxy,
		proxy_iBUF, sizeof proxy_iBUF);
	buffer_init(&proxy_out, (int(*)())write, fd_proxy,
		proxy_oBUF, sizeof proxy_oBUF);

	/* send the request to squid */
	io_wantwrite(fd_proxy); io_wait(); io_dontwantwrite(fd_proxy);
	buffer_puts(&proxy_out, squid_method);
	buffer_puts(&proxy_out, " ");
	buffer_puts(&proxy_out, squid_url);
	buffer_puts(&proxy_out, " HTTP/1.0\r\n");
	buffer_puts(&proxy_out, "User-Agent: " VERSION);
	buffer_puts(&proxy_out, "\r\n\r\n");
	buffer_flush(&proxy_out);

	/* look for bad content (if wanted) */
	cfilter_todo = SCAN_UNKNOWN;
	pfilter_todo = SCAN_UNKNOWN;
	pfilter_current = 0;
	io_wantread(fd_proxy); io_wait(); io_dontwantread(fd_proxy);
	while (buffer_get(&proxy_in, &ch, 1) == 1) {
		if (++len >= maxsize) break;

		/* send it to clamd */
		if (isset(todo, TODO_VIRUS)) {
			check_clamav_put2stream(ch);
		}

		if (!stralloc_catb(&content, &ch, 1)) die_nomem();
		if (ch == '\n') { /* 'HTTP/1.0 200 OK\r\n' */
			if (content.len < 2) {
				if (!stralloc_copys(&content, "")) die_nomem();
				continue;
			}
			content.s[content.len-2]=0;

			if (cfilter_todo == SCAN_UNKNOWN) {
				cfilter_todo = check_cont_hdr(&array_cfilter_todo, content.s);
			}

			if (pfilter_todo == SCAN_UNKNOWN) {
				pfilter_todo = check_cont_hdr(&array_pfilter_todo, content.s);
			}

			if ((check_cont_line(content.s)) == ACL_DENY) {
				/* close proxy stuff */
				proxy_out.fd=-1; buffer_close(&proxy_out); io_close(fd_proxy);
				stralloc_free(&content);
				stralloc_free(&pfilter_matches);
				return ACL_DENY;
			}

			if (!stralloc_copys(&content, "")) die_nomem();
		}
	}

	/* close proxy stuff */
	proxy_out.fd=-1; buffer_close(&proxy_out); io_close(fd_proxy);
	stralloc_free(&content);
	stralloc_free(&pfilter_matches);

	/* see, what clamav means to our stream */
	if (isset(todo, TODO_VIRUS)) {
		if (check_clamav_checkresult()) {
			/* log the virus site */
			log_headline("virus");
			log_sa(&url);    log_s("\t");
			log_sa(&ip);     log_s("\t");
			log_sa(&host);   log_s("\t");
			log_sa(&user);   log_s("\t");
			log_sa(&method); log_s("\t");
			log_sa(&virus);  log_s("\n");
			log_flush();
			redir_site(REDIR_TYPE_VIRUS);
			return ACL_DENY;
		}
	}

	return ACL_ALLOW;
}
