#ifndef SW_PRAMS_H
#define SW_PRAMS_H

/**
 * all these values can be overridden by environment variables!
 */

/* this logfile will show you the denied users/hosts/pages */
#define LOG_FILE         "/var/log/squid/squidwall.log"

/**
 * ip:port of your squid cache
 *
 * warning: this cache should not re-redirect to the redirector :)
 */
#define PROXY            "127.0.0.1:3128"

/* ip:port of clamd in TCP/IP mode */
#define CLAMD            "127.0.0.1:3310"

/* /some/path/to/clamd.sock of clamd in local socket mode */
#define CLAMAV_SOCKET    "/var/lib/clamav/clamd.sock"

/**
 * the default size in bytes, which will be passed through the content
 * filter and the clamav stream for antivirus checking
 */
#define MAX_SIZE         60000

/* phrase filter */
#define PFILTER_MAX      100 /* deadline for the pfilters, variable: 0..255 */
#define PFILTER_DEFAULT  25  /* for unacceptable values in configuration */

/* user / host acl's and redirects */
#define CONFIG_DIR        "/etc/squidwall"
#define HOSTS_DIR         CONFIG_DIR "/hosts"
#define USERS_DIR         CONFIG_DIR "/users"
#define REDIR_DIR         CONFIG_DIR "/redirects"

/* content filters */
#define PFILTERS_DIR      CONFIG_DIR "/filters/pfilter"
#define CFILTERS_DIR      CONFIG_DIR "/filters/cfilter"
#define CRFILTERS_DIR     CONFIG_DIR "/filters/crfilter"
#define BCFILTERS_DIR     CONFIG_DIR "/filters/bcfilter"

/* url filters */
#define BFILTERS_DIR      CONFIG_DIR "/filters/bfilter"
#define BRFILTERS_DIR     CONFIG_DIR "/filters/brfilter"
#define UFILTERS_DIR      CONFIG_DIR "/filters/ufilter"
#define URFILTERS_DIR     CONFIG_DIR "/filters/urfilter"
#define WFILTERS_DIR      CONFIG_DIR "/filters/wfilter"
#define WRFILTERS_DIR     CONFIG_DIR "/filters/wrfilter"
#define UFILTERS_MD5_DIR  CONFIG_DIR "/filters/ufilter_md5"
#define UFILTERS_SHA1_DIR CONFIG_DIR "/filters/ufilter_sha1"

/* url forbidden */
#define REDIR_BADURL     "http://www.example.org/?network=x&reason=url&"

/* forbidden content */
#define REDIR_BADCONT    "http://www.example.org/?network=x&reason=cont&"

/* host/ip forbidden */
#define REDIR_BADHOST    "http://www.example.org/?network=x&reason=host&"

/* user forbidden */
#define REDIR_BADUSER    "http://www.example.org/?network=x&reason=user&"

/* if a virus is found, the user will be redirected to this page */
#define REDIR_VIRUSFOUND "http://www.example.org/?network=x&reason=virus&"

/* if a banner-url is found, the user will be redirected to this image */
#define REDIR_BANNERFOUND "http://www.example.org/img/t.png"

/* this url nerver redirected! */
#define REDIR_SAFESITE   "http://www.example.org/"

/* this url is used for configuration via http://squidwall */
#define REDIR_WEBINTERFACE "http://www.example.org/inet/"

/* if unneeded, define it to zero */
#define WANT_STARTUP_INFO 1

/* debugging of some functions / files */
#define DEBUG_SQUIDWALL      1
#define DEBUG_READ_ACLS      2
#define DEBUG_READ_FILTERS   4
#define DEBUG_CHECK_HOST     8
#define DEBUG_CHECK_USER    16
#define DEBUG_CHECK_BANNER  32
#define DEBUG_CHECK_WLIST   64
#define DEBUG_CHECK_BLIST  128
#define DEBUG_CHECK_CONT   256
#define DEBUG_CHECK_REDIR  512
#define DEBUG_CHECK_VIRUS 1024

#define DEBUG_ALL   (1|2|4|16|32|64|128|256|512|1024)
//#define DEBUG_LEVEL       (DEBUG_SQUIDWALL|DEBUG_READ_ACLS|DEBUG_CHECK_USER|DEBUG_CHECK_HOST|DEBUG_CHECK_CONT)
//#define DEBUG_LEVEL DEBUG_ALL
//#define DEBUG_LEVEL (DEBUG_CHECK_VIRUS|DEBUG_CHECK_BLIST)

#endif
