#include "squidwall.h"

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * build a redirection url for squid
 *
 * version  => version of squidwall
 * ip       => ip address of the client
 * url      => requested url
 * host     => hostname of the client
 * user     => username (via identd of the client)
 * method   => request method (GET/POST/...)
 *
 * only, if virus found:
 * clamd    => version of clamav
 * virus    => name of virus
 */

stralloc encoded_ip;
stralloc encoded_url;
stralloc encoded_host;
stralloc encoded_user;
stralloc encoded_method;
stralloc encoded_version;

stralloc encoded_virus;
stralloc encoded_clamav;
stralloc encoded_filter;

void redir_site(int type)
{
	char *site=0;

	/* init the stralloc's */
	stralloc_init(&encoded_ip);
	stralloc_init(&encoded_url);
	stralloc_init(&encoded_host);
	stralloc_init(&encoded_user);
	stralloc_init(&encoded_method);
	stralloc_init(&encoded_version);

	/* change the url for squid */
	if (!fmt_to_sa(fmt_urlencoded, &encoded_ip,
				ip.s, ip.len)) die_nomem();
	if (!fmt_to_sa(fmt_urlencoded, &encoded_url,
				url.s, url.len)) die_nomem();
	if (!fmt_to_sa(fmt_urlencoded, &encoded_host,
				host.s, host.len)) die_nomem();
	if (!fmt_to_sa(fmt_urlencoded, &encoded_user,
				user.s, user.len)) die_nomem();
	if (!fmt_to_sa(fmt_urlencoded, &encoded_method,
				method.s, method.len)) die_nomem();
	if (!fmt_to_sa(fmt_urlencoded, &encoded_version,
				VERSION, sizeof(VERSION)-1)) die_nomem();

	switch (type) {
	case REDIR_TYPE_USER:
		site=redir_baduser;
		break;
	case REDIR_TYPE_HOST:
		site=redir_badhost;
		break;
	case REDIR_TYPE_URL:
		site=redir_badurl;
		stralloc_init(&encoded_filter);
		if (!fmt_to_sa(fmt_urlencoded, &encoded_filter,
					filter.s, filter.len)) die_nomem();
		break;
	case REDIR_TYPE_CONT:
		site=redir_badcont;
		stralloc_init(&encoded_filter);
		if (!fmt_to_sa(fmt_urlencoded, &encoded_filter,
					filter.s, filter.len)) die_nomem();
		break;
	case REDIR_TYPE_VIRUS:
		site=redir_virusfound;
		stralloc_init(&encoded_virus);
		stralloc_init(&encoded_clamav);
		if (!fmt_to_sa(fmt_urlencoded, &encoded_virus,
					virus.s, virus.len)) die_nomem();
		if (!fmt_to_sa(fmt_urlencoded, &encoded_clamav,
					clamav.s, clamav.len)) die_nomem();
		break;
	case REDIR_TYPE_BANNER:
		if (!stralloc_copys(&redirsite, redir_bannerfound)) die_nomem();
		return;
	case REDIR_TYPE_CONFIG:
		if (!stralloc_copys(&redirsite, redir_webinterface)) die_nomem();
		if (!stralloc_cats(&redirsite, squid_url+16)) die_nomem();
		return;
	}

	/* [www.example.org/?network=...&]clamd=...&ip=...&... */
	if (!stralloc_copys(&redirsite, site)) die_nomem();

	if (type == REDIR_TYPE_VIRUS) {
		if (!stralloc_cats(&redirsite, "clamd=")) die_nomem();
		if (!stralloc_cat(&redirsite, &encoded_clamav)) die_nomem();
		if (!stralloc_cats(&redirsite, "&virus=")) die_nomem();
		if (!stralloc_cat(&redirsite, &encoded_virus)) die_nomem();
		if (!stralloc_cats(&redirsite, "&")) die_nomem();

		stralloc_free(&encoded_virus);
		stralloc_free(&encoded_clamav);
	}

	if (type == REDIR_TYPE_URL || type == REDIR_TYPE_CONT) {
		if (!stralloc_cats(&redirsite, "filter=")) die_nomem();
		if (!stralloc_cat(&redirsite, &encoded_filter)) die_nomem();
		if (!stralloc_cats(&redirsite, "&")) die_nomem();

		stralloc_free(&encoded_filter);
	}

	if (!stralloc_cats(&redirsite, "version=")) die_nomem();
	if (!stralloc_cat(&redirsite, &encoded_version)) die_nomem();
	if (!stralloc_cats(&redirsite, "&ip=")) die_nomem();
	if (!stralloc_cat(&redirsite, &encoded_ip)) die_nomem();
	if (!stralloc_cats(&redirsite, "&url=")) die_nomem();
	if (!stralloc_cat(&redirsite, &encoded_url)) die_nomem();
	if (!stralloc_cats(&redirsite, "&host=")) die_nomem();
	if (!stralloc_cat(&redirsite, &encoded_host)) die_nomem();
	if (!stralloc_cats(&redirsite, "&user=")) die_nomem();
	if (!stralloc_cat(&redirsite, &encoded_user)) die_nomem();
	if (!stralloc_cats(&redirsite, "&method=")) die_nomem();
	if (!stralloc_cat(&redirsite, &encoded_method)) die_nomem();
	if (!stralloc_cats(&redirsite, " ")) die_nomem();
	if (!stralloc_cat(&redirsite, &encoded_ip)) die_nomem();
	if (!stralloc_cats(&redirsite, "/")) die_nomem();
	if (!stralloc_cat(&redirsite, &encoded_host)) die_nomem();
	if (!stralloc_cats(&redirsite, " ")) die_nomem();
	if (!stralloc_cat(&redirsite, &encoded_user)) die_nomem();
	if (!stralloc_cats(&redirsite, " ")) die_nomem();
	if (!stralloc_cat(&redirsite, &encoded_method)) die_nomem();

	stralloc_free(&encoded_version);
	stralloc_free(&encoded_ip);
	stralloc_free(&encoded_url);
	stralloc_free(&encoded_host);
	stralloc_free(&encoded_user);
	stralloc_free(&encoded_method);

	return;
}
